package sun.awt.image;

import java.awt.image.ImageObserver;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.List;
import java.util.Arrays;
import java.awt.Dimension;
import java.awt.Image;
import java.util.function.BiFunction;
import java.awt.geom.Dimension2D;

public class MultiResolutionCachedImage extends AbstractMultiResolutionImage
{
    private final int baseImageWidth;
    private final int baseImageHeight;
    private final Dimension2D[] sizes;
    private final BiFunction<Integer, Integer, Image> mapper;
    private int availableInfo;
    
    public MultiResolutionCachedImage(int width, int height, BiFunction<Integer, Integer, Image> biFunction) {
        this(width, height, new Dimension[] { new Dimension(width, height) }, biFunction);
    }
    
    public MultiResolutionCachedImage(int baseImageWidth, int baseImageHeight, Dimension2D[] original, BiFunction<Integer, Integer, Image> mapper) {
        this.baseImageWidth = baseImageWidth;
        this.baseImageHeight = baseImageHeight;
        this.sizes = ((original == null) ? null : ((Dimension2D[])Arrays.copyOf(original, original.length)));
        this.mapper = mapper;
    }
    
    @Override
    public Image getResolutionVariant(int i, int j) {
        ImageCache instance = ImageCache.getInstance();
        MultiResolutionCachedImage.ImageCacheKey imageCacheKey = new MultiResolutionCachedImage.ImageCacheKey(this, i, j);
        Image image = instance.getImage(imageCacheKey);
        if (image == null) {
            image = (Image)this.mapper.apply(Integer.valueOf(i), Integer.valueOf(j));
            instance.setImage(imageCacheKey, image);
        }
        preload(image, this.availableInfo);
        return image;
    }
    
    @Override
    public List<Image> getResolutionVariants() {
        return (List)Arrays.stream(this.sizes).map(dimension2D/* java.awt.geom.Dimension2D, */ -> this.getResolutionVariant((int)dimension2D.getWidth(), (int)dimension2D.getHeight())).collect(Collectors.toList());
    }
    
    public MultiResolutionCachedImage map(Function<Image, Image> function) {
        return new MultiResolutionCachedImage(this.baseImageWidth, this.baseImageHeight, this.sizes, (n, n2)/* java.lang.Integer,java.lang.Integer, */ -> (Image)p0.apply(this.getResolutionVariant((int)n, (int)n2)));
    }
    
    @Override
    public int getWidth(ImageObserver imageObserver) {
        this.updateInfo(imageObserver, 1);
        return super.getWidth(imageObserver);
    }
    
    @Override
    public int getHeight(ImageObserver imageObserver) {
        this.updateInfo(imageObserver, 2);
        return super.getHeight(imageObserver);
    }
    
    @Override
    public Object getProperty(String s, ImageObserver imageObserver) {
        this.updateInfo(imageObserver, 4);
        return super.getProperty(s, imageObserver);
    }
    
    @Override
    protected Image getBaseImage() {
        return this.getResolutionVariant(this.baseImageWidth, this.baseImageHeight);
    }
    
    private void updateInfo(ImageObserver imageObserver, int n) {
        this.availableInfo |= ((imageObserver == null) ? 32 : n);
    }
    
    private static int getInfo(Image image) {
        if ((image instanceof ToolkitImage)) {
            return ((ToolkitImage)image).getImageRep().check((p0, p1, p2, p3, p4, p5)/* java.awt.Image,int,int,int,int,int, */ -> false);
        }
        return 0;
    }
    
    private static void preload(Image image, final int n) {
        if (n != 0 && image instanceof ToolkitImage) {
            ((ToolkitImage)image).preload(new ImageObserver() {
                int flags = n;
                
                @Override
                public boolean imageUpdate(Image image, int n, int n2, int n3, int n4, int n5) {
                    this.flags &= ~n;
                    return this.flags != 0 && (n & 0xC0) == 0x0;
                }
            });
        }
    }
    
    private static class ImageCacheKey implements ImageCache.PixelsKey
    {
        private final int pixelCount;
        private final int hash;
        private final int w;
        private final int h;
        private final Image baseImage;
        
        ImageCacheKey(Image baseImage, int w, int h) {
            this.baseImage = baseImage;
            this.w = w;
            this.h = h;
            this.pixelCount = w * h;
            this.hash = this.hash();
        }
        
        @Override
        public int getPixelCount() {
            return this.pixelCount;
        }
        
        private int hash() {
            return 31 * (31 * this.baseImage.hashCode() + this.w) + this.h;
        }
        
        @Override
        public int hashCode() {
            return this.hash;
        }
        
        @Override
        public boolean equals(Object o) {
            if ((o instanceof MultiResolutionCachedImage.ImageCacheKey)) {
                MultiResolutionCachedImage.ImageCacheKey imageCacheKey = (MultiResolutionCachedImage.ImageCacheKey)o;
                return this.baseImage == imageCacheKey.baseImage && this.w == imageCacheKey.w && this.h == imageCacheKey.h;
            }
            return false;
        }
    }
}
